Package: multinma
Title: Bayesian Network Meta-Analysis of Individual and Aggregate Data
Version: 0.9.1
Authors@R: 
    c(person(given = c("David", "M."),
           family = "Phillippo",
           role = c("aut", "cre"),
           email = "david.phillippo@bristol.ac.uk",
           comment = c(ORCID = "0000-0003-2672-7841")),
    person(given = c("Samuel", "J."),
            family = "Perren",
            role = "ctb",
            email = "samuel.perren@bristol.ac.uk",
            comment = c(ORCID = "0009-0005-1921-6957")),
    person(given = "Niels",
            family = "Dunnewind",
            role = "ctb",
            comment = c(ORCID = "0009-0009-8675-9109")))
Description: Network meta-analysis and network meta-regression models for 
    aggregate data, individual patient data, and mixtures of both individual 
    and aggregate data using multilevel network meta-regression as described by
    Phillippo et al. (2020) <doi:10.1111/rssa.12579>. Models are estimated in a
    Bayesian framework using 'Stan'.
License: GPL-3
Encoding: UTF-8
LazyData: true
Biarch: true
Depends: R (>= 3.4.0)
Imports: methods, stats, graphics, survival, Rcpp (>= 0.12.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), rstantools (>=
        2.0.0), Rdpack (>= 0.7), tibble (>= 2.1.3), dplyr (>= 1.0.0),
        rlang, purrr, forcats, glue, randtoolbox, copula, tidyr (>=
        1.0.0), stringr, Matrix, igraph, ggraph, ggplot2 (>= 3.3.0),
        ggdist (>= 2.1.1), truncdist, bayesplot
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>=
        2.32.9)
SystemRequirements: GNU make
RoxygenNote: 7.3.3
Suggests: testthat (>= 2.1.0), vdiffr, withr, knitr, rmarkdown,
        parallel, R.rsp, rprojroot, loo (>= 2.0.0), crayon, tidygraph,
        pkgdown, splines2 (>= 0.5.0), flexsurv, rstpm2
RdMacros: Rdpack
URL: https://dmphillippo.github.io/multinma/,
        https://github.com/dmphillippo/multinma
BugReports: https://github.com/dmphillippo/multinma/issues
VignetteBuilder: knitr, R.rsp
NeedsCompilation: yes
Packaged: 2026-04-16 09:58:05 UTC; dp14189
Author: David M. Phillippo [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2672-7841>),
  Samuel J. Perren [ctb] (ORCID: <https://orcid.org/0009-0005-1921-6957>),
  Niels Dunnewind [ctb] (ORCID: <https://orcid.org/0009-0009-8675-9109>)
Maintainer: David M. Phillippo <david.phillippo@bristol.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-16 23:10:09 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-20 23:47:17 UTC; unix
Archs: multinma.so.dSYM
