# Generated by roxygen2: do not edit by hand

S3method("names<-",mcmc_array)
S3method(add_integration,data.frame)
S3method(add_integration,default)
S3method(add_integration,nma_data)
S3method(as.array,nma_dic)
S3method(as.array,nma_rank_probs)
S3method(as.array,nma_summary)
S3method(as.array,stan_nma)
S3method(as.data.frame,nma_dic)
S3method(as.data.frame,nma_summary)
S3method(as.data.frame,nodesplit_summary)
S3method(as.data.frame,stan_nma)
S3method(as.igraph,nma_data)
S3method(as.matrix,nma_dic)
S3method(as.matrix,nma_rank_probs)
S3method(as.matrix,nma_summary)
S3method(as.matrix,stan_nma)
S3method(as.stanfit,default)
S3method(as.stanfit,stan_nma)
S3method(as.tibble,nma_dic)
S3method(as.tibble,nma_summary)
S3method(as.tibble,stan_nma)
S3method(as_tibble,nma_dic)
S3method(as_tibble,nma_summary)
S3method(as_tibble,nodesplit_summary)
S3method(as_tibble,stan_nma)
S3method(names,mcmc_array)
S3method(pairs,stan_nma)
S3method(plot,mcmc_array)
S3method(plot,nma_data)
S3method(plot,nma_dic)
S3method(plot,nma_nodesplit)
S3method(plot,nma_nodesplit_df)
S3method(plot,nma_parameter_summary)
S3method(plot,nma_rank_probs)
S3method(plot,nma_summary)
S3method(plot,nodesplit_summary)
S3method(plot,stan_nma)
S3method(plot,surv_nma_summary)
S3method(predict,stan_nma)
S3method(predict,stan_nma_surv)
S3method(print,mcmc_array)
S3method(print,mlnmr_data)
S3method(print,nma_data)
S3method(print,nma_dic)
S3method(print,nma_nodesplit)
S3method(print,nma_nodesplit_df)
S3method(print,nma_prior)
S3method(print,nma_summary)
S3method(print,nodesplit_summary)
S3method(print,stan_nma)
S3method(stats::knots,stan_nma)
S3method(summary,mcmc_array)
S3method(summary,nma_nodesplit)
S3method(summary,nma_nodesplit_df)
S3method(summary,nma_prior)
S3method(summary,stan_nma)
export(.default)
export(.is_default)
export(Hmspline)
export(RE_cor)
export(Surv)
export(add_integration)
export(as.stanfit)
export(as.tibble.nodesplit_summary)
export(cauchy)
export(combine_network)
export(dbern)
export(dgamma)
export(dgent)
export(dic)
export(distr)
export(dlogitnorm)
export(dlogt)
export(dmspline)
export(exponential)
export(flat)
export(geom_km)
export(get_nodesplits)
export(half_cauchy)
export(half_normal)
export(half_student_t)
export(has_direct)
export(has_indirect)
export(hmspline)
export(inv_softmax)
export(is_network_connected)
export(log_normal)
export(log_student_t)
export(make_knots)
export(marginal_effects)
export(multi)
export(nma)
export(normal)
export(pbern)
export(pgamma)
export(pgent)
export(plogitnorm)
export(plogt)
export(plot_integration_error)
export(plot_prior_posterior)
export(pmspline)
export(posterior_rank_probs)
export(posterior_ranks)
export(qbern)
export(qgamma)
export(qgent)
export(qlogitnorm)
export(qlogt)
export(qmspline)
export(relative_effects)
export(rmst_mspline)
export(set_agd_arm)
export(set_agd_contrast)
export(set_agd_surv)
export(set_ipd)
export(softmax)
export(student_t)
export(theme_multinma)
export(unnest_integration)
export(which_RE)
import(Rcpp)
import(methods)
importFrom(RcppParallel,CxxFlags)
importFrom(Rdpack,reprompt)
importFrom(dplyr,"%>%")
importFrom(grDevices,nclass.Sturges)
importFrom(graphics,pairs)
importFrom(graphics,plot)
importFrom(igraph,as.igraph)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,enquo)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,dbinom)
importFrom(stats,dnorm)
importFrom(stats,dunif)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,optim)
importFrom(stats,pbinom)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,qbinom)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(tibble,as.tibble)
importFrom(tibble,as_tibble)
importFrom(truncdist,dtrunc)
importFrom(truncdist,ptrunc)
importFrom(truncdist,qtrunc)
importFrom(utils,head)
importFrom(utils,packageVersion)
useDynLib(multinma, .registration = TRUE)
