% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{as.stanfit}
\alias{as.stanfit}
\alias{as.stanfit.stan_nma}
\alias{as.stanfit.default}
\title{as.stanfit}
\usage{
as.stanfit(x, ...)

\method{as.stanfit}{stan_nma}(x, ...)

\method{as.stanfit}{default}(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{additional arguments}
}
\value{
A \code{\link[rstan:stanfit-class]{stanfit}} object.
}
\description{
Attempt to turn an object into a \code{\link[rstan:stanfit-class]{stanfit}} object.
}
