% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodesplit_summary-class.R
\name{print.nodesplit_summary}
\alias{print.nodesplit_summary}
\alias{as_tibble.nodesplit_summary}
\alias{as.tibble.nodesplit_summary}
\alias{as.data.frame.nodesplit_summary}
\title{Methods for \code{nodesplit_summary} objects}
\usage{
\method{print}{nodesplit_summary}(x, ..., digits = 2)

\method{as_tibble}{nodesplit_summary}(x, ..., nest = FALSE)

as.tibble.nodesplit_summary(x, ..., nest = FALSE)

\method{as.data.frame}{nodesplit_summary}(x, ...)
}
\arguments{
\item{x}{A \code{nodesplit_summary} object}

\item{...}{Additional arguments passed on to other methods}

\item{digits}{Integer number of digits to display}

\item{nest}{Whether to return a nested tibble, with the full \link{nma_summary}
and \link{nma_dic} objects, or to unnest their summaries, default \code{FALSE}}
}
\value{
A \code{data.frame} for \code{as.data.frame()}, a \code{tbl_df} for \code{as.tibble()}
and \code{as_tibble()}.

The \code{print()} method returns \code{x} invisibly.
}
\description{
The \code{as.data.frame()}, \code{as_tibble()}, and \code{as.tibble()} methods return the
node-splitting summaries in a data frame or tibble.
}
\seealso{
\code{\link[=plot.nodesplit_summary]{plot.nodesplit_summary()}}
}
