% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{smoking}
\alias{smoking}
\title{Smoking cessation data}
\format{
A data frame with 50 rows and 5 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
}
}
\usage{
smoking
}
\description{
Data frame containing the results of 24 trials of 4 smoking cessation
treatments \insertCite{Hasselblad1998,TSD4}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
