% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/default.R
\name{.default}
\alias{.default}
\alias{.is_default}
\title{Set default values}
\usage{
.default(x = list())

.is_default(x)
}
\arguments{
\item{x}{An object}
}
\value{
For \code{.default()}, an identical object with additional attribute
\code{.default}. For \code{.is_default()}, a logical value (\code{TRUE} or \code{FALSE}).
}
\description{
The \code{.default()} function is used internally to mark certain values as
default, so that the user may be notified when default values are being used.
For example, choosing a default reference treatment for a network, or using
default prior distributions. The function \code{.is_default()} checks whether an
argument/object is set to a default value. Neither of these functions are
intended to be called by the user.
}
