% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{atrial_fibrillation}
\alias{atrial_fibrillation}
\title{Stroke prevention in atrial fibrillation patients}
\format{
A data frame with 63 rows and 11 variables:
\describe{
\item{studyc}{study name}
\item{studyn}{numeric study ID}
\item{trtc}{treatment name}
\item{trtn}{numeric treatment code}
\item{trt_class}{treatment class}
\item{r}{number of events}
\item{n}{sample size}
\item{E}{person-years at risk}
\item{stroke}{proportion of individuals with prior stroke}
\item{year}{year of study publication}
\item{followup}{mean length of follow-up (years)}
}
}
\usage{
atrial_fibrillation
}
\description{
Data frame containing the results of 26 trials comparing 17 treatments in 4
classes for the prevention of stroke in patients with atrial fibrillation
\insertCite{Cooper2009}{multinma}. The data are the corrected versions
given by \insertCite{gemtc;textual}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
