% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{social_anxiety}
\alias{social_anxiety}
\title{Social Anxiety}
\format{
A data frame with 248 rows and 8 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{treatment ID}
\item{y}{standardised mean difference of the outcome measure}
\item{se}{standard error of the outcome measure}
\item{trtc}{treatment name}
\item{classn}{class ID}
\item{classc}{class name}
}
}
\usage{
social_anxiety
}
\description{
Data frame containing the results of 101 clinical trials comparing 41 first-line treatments for social anxiety disorder
in adults . The 41 treatments are further categorised into 17 distinct classes. The interventions of interest include oral drugs,
psychological or behavioural therapies, and combinations of pharmacological and psychological therapies.
Following Mayo-Wilson et al. \insertCite{mayo2014psychological}{multinma} the data are given as standardised mean differences (SMD) to allow comparison across the different scales used in each study.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
