% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_data-class.R
\name{print.nma_data}
\alias{print.nma_data}
\alias{print.mlnmr_data}
\title{Print \code{nma_data} objects}
\usage{
\method{print}{nma_data}(x, ..., n = 10)

\method{print}{mlnmr_data}(x, ..., n = 10)
}
\arguments{
\item{x}{\code{nma_data} object}

\item{...}{other options (not used)}

\item{n}{number of studies of each type to print}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print details of networks stored as \link{nma_data} objects, as created by
\code{\link[=set_ipd]{set_ipd()}}, \code{\link[=set_agd_arm]{set_agd_arm()}}, \code{\link[=set_agd_contrast]{set_agd_contrast()}}, \code{\link[=set_agd_surv]{set_agd_surv()}}, or
\code{\link[=combine_network]{combine_network()}}.
}
