% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_nodesplit-class.R
\name{print.nma_nodesplit_df}
\alias{print.nma_nodesplit_df}
\alias{print.nma_nodesplit}
\title{Print \code{nma_nodesplit_df} objects}
\usage{
\method{print}{nma_nodesplit_df}(x, ...)

\method{print}{nma_nodesplit}(x, ...)
}
\arguments{
\item{x}{A \link{nma_nodesplit_df} object}

\item{...}{Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print \code{nma_nodesplit_df} objects
}
\seealso{
The summary method \code{\link[=summary.nma_nodesplit_df]{summary.nma_nodesplit_df()}} summarises the
node-splitting results.
}
