% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dietary_fat}
\alias{dietary_fat}
\title{Reduced dietary fat to prevent mortality}
\format{
A data frame with 21 rows and 7 variables:
\describe{
\item{studyn}{numeric study ID}
\item{studyc}{study name}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{number of events}
\item{n}{number randomised}
\item{E}{person-years at risk}
}
}
\usage{
dietary_fat
}
\description{
Data frame containing the number of deaths and person-years at risk in 10
trials comparing reduced fat diets vs. control (non-reduced fat diet) for
preventing mortality \insertCite{Hooper2000,TSD2}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
