% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blocker}
\alias{blocker}
\title{Beta blockers to prevent mortality after MI}
\format{
A data frame with 44 rows and 5 variables:
\describe{
\item{studyn}{numeric study ID}
\item{trtn}{numeric treatment code}
\item{trtc}{treatment name}
\item{r}{total number of events}
\item{n}{total number of individuals}
}
}
\usage{
blocker
}
\description{
Data frame containing the number of deaths in 22 trials comparing beta
blockers vs. control for preventing mortality after myocardial infarction
\insertCite{Carlin1992,TSD2}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
