% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{loo.stan_nma}
\alias{loo.stan_nma}
\alias{loo}
\alias{waic.stan_nma}
\alias{waic}
\title{Model comparison using the \code{loo} package}
\usage{
\method{loo}{stan_nma}(x, ...)

\method{waic}{stan_nma}(x, ...)
}
\arguments{
\item{x}{An object of class \link{stan_nma} or \link{stan_mlnmr}}

\item{...}{Further arguments to \code{\link[rstan:stanfit-method-loo]{loo()}} or
\code{\link[loo:waic]{waic()}}}
}
\description{
The \code{\link[loo:loo]{loo()}} and \code{\link[loo:waic]{waic()}} functions from the \code{loo}
package may be called directly on \link{stan_nma} and \link{stan_mlnmr} objects.
}
