% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{dgent}
\alias{dgent}
\alias{pgent}
\alias{qgent}
\title{Generalised Student's t distribution (with location and scale)}
\usage{
dgent(x, df, location = 0, scale = 1)

pgent(q, df, location = 0, scale = 1)

qgent(p, df, location = 0, scale = 1)
}
\arguments{
\item{x, q}{Vector of quantiles}

\item{df}{Degrees of freedom, greater than zero}

\item{location}{Location parameter}

\item{scale}{Scale parameter, greater than zero}

\item{p}{Vector of probabilities}
}
\value{
\code{dgent()} gives the density, \code{pgent()} gives the distribution
function, \code{qgent()} gives the quantile function.
}
\description{
Density, distribution, and quantile function for the generalised t
distribution with degrees of freedom \code{df}, shifted by \code{location} and scaled
by \code{scale}.
}
