% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nma_dic-class.R
\name{nma_dic-class}
\alias{nma_dic-class}
\alias{nma_dic}
\title{The nma_dic class}
\description{
The \code{nma_dic} class contains details of the Deviance Information Criterion
(DIC), produced using the \code{\link[=dic]{dic()}} function.
}
\details{
Objects of class \code{nma_dic} have the following components:
\describe{
\item{\code{dic}}{The DIC value}
\item{\code{pd}, \code{pv}}{The effective number of parameters}
\item{\code{resdev}}{The total residual deviance}
\item{\code{pointwise}}{A list of data frames containing the pointwise
contributions for the IPD and AgD.}
\item{\code{resdev_array}}{A 3D MCMC array [Iterations, Chains, Parameters] of
posterior residual deviance samples.}
}
}
\seealso{
\code{\link[=dic]{dic()}}, \code{\link[=print.nma_dic]{print.nma_dic()}}, \code{\link[=plot.nma_dic]{plot.nma_dic()}}.
}
