% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_multinma.R
\name{theme_multinma}
\alias{theme_multinma}
\title{Plot theme for multinma plots}
\usage{
theme_multinma(...)
}
\arguments{
\item{...}{Arguments passed to
\code{\link[ggplot2:ggtheme]{ggplot2::theme_light()}}}
}
\value{
A \code{ggplot2} theme
}
\description{
A simple \code{ggplot2} theme for plots in the \code{multinma} package.
}
\examples{
library(ggplot2)
theme_set(theme_multinma())

}
\seealso{
\code{\link[ggplot2:theme]{ggplot2::theme()}}, \code{\link[ggplot2:theme_get]{ggplot2::theme_set()}}
}
