% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mspline.R
\name{knots.stan_nma}
\alias{knots.stan_nma}
\title{Knot locations for a fitted model}
\usage{
\method{knots}{stan_nma}(Fn, type = c("all", "internal", "boundary"), ...)
}
\arguments{
\item{Fn}{A fitted \link{stan_nma} object}

\item{type}{String, indicating whether to return all knots (\code{"all"}, the
default), or only the internal knots (\code{"internal"}) or boundary knots
(\code{"boundary"}).}

\item{...}{Other arguments, passed on to \code{\link[splines2:knots]{splines2::knots()}}}
}
\value{
A list of vectors of knot locations, for each study in the network.
}
\description{
Obtain the knot locations from a fitted M-spline or piecewise exponential
model.
}
\seealso{
\code{\link[=make_knots]{make_knots()}} for constructing knots for a network object.
}
