% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{transfusion}
\alias{transfusion}
\title{Granulocyte transfusion in patients with neutropenia or neutrophil
dysfunction}
\format{
A data frame with 12 rows and 4 variables:
\describe{
\item{studyc}{study name}
\item{trtc}{treatment name}
\item{r}{total number of deaths}
\item{n}{total number of individuals}
}
}
\usage{
transfusion
}
\description{
Data frame containing the number of deaths in 6 trials comparing transfusion
of granulocytes (white blood cells) to control
\insertCite{Stanworth2005}{multinma}. Previously used to demonstrate
informative prior distributions for the heterogeneity variance by
\insertCite{Turner2012;textual}{multinma}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
