% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_nma-class.R
\name{print.stan_nma}
\alias{print.stan_nma}
\title{Print \code{stan_nma} objects}
\usage{
\method{print}{stan_nma}(x, ...)
}
\arguments{
\item{x}{A \link{stan_nma} object}

\item{...}{Further arguments passed to \code{\link[rstan:print.stanfit]{print.stanfit()}}}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print \code{stan_nma} objects
}
