% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multinma-package.R
\docType{package}
\name{multinma-package}
\alias{multinma-package}
\alias{multinma}
\title{multinma: A Package for Network Meta-Analysis of Individual and Aggregate
Data in Stan}
\description{
\if{html}{\figure{logo.svg}{options: width="120" style="float: right;" alt="multinma logo"}}
An R package for performing network meta-analysis and network meta-regression
with aggregate data, individual patient data, or mixtures of both.
}
\details{
Network meta-analysis (NMA) combines (aggregate) data from multiple
studies on multiple treatments in order to produce consistent estimates of
relative treatment effects between each pair of treatments in the network
\insertCite{TSD2}{multinma}.

Network meta-regression (NMR) extends NMA to include covariates, allowing
adjustment for differences in effect-modifying variables between studies
\insertCite{TSD3}{multinma}. NMR is typically performed using aggregate
data (AgD), which lacks power and is prone to ecological bias. NMR with
individual patient data (IPD) is the gold standard, if data are available.

Multilevel network meta-regression (ML-NMR) allows IPD and AgD to be
incorporated together in a network meta-regression
\insertCite{methods_paper,Phillippo_thesis}{multinma}. As in IPD NMR, an
individual-level regression model is defined. AgD studies are then fitted
by integrating the individual-level model over the respective covariate
distributions. This correctly links the two levels of the model (instead of
"plugging in" mean covariate values), avoiding aggregation bias.
Population-adjusted treatment effects \insertCite{TSD18}{multinma} can be
produced for any study population in the network, or for an external target
population.

Models are estimated in a Bayesian framework using Stan
\insertCite{Carpenter2017}{multinma}. Quasi-Monte Carlo numerical
integration based on Sobol' sequences is used for the integration in ML-NMR
models, with a Gaussian copula to account for correlations between
covariates \insertCite{methods_paper,Phillippo_thesis}{multinma}.
}
\section{Getting Started}{

A good place to start is with the package vignettes which walk through
example analyses, see \code{vignette("vignette_overview")} for an overview.
The series of NICE Technical Support Documents on evidence synthesis gives
a detailed introduction to network meta-analysis:

\insertRef{TSD_evsynth}{multinma}

Multilevel network meta-regression is set out in the following methods paper:

\insertRef{methods_paper}{multinma}
}

\references{
\insertAllCited{}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dmphillippo.github.io/multinma/}
  \item \url{https://github.com/dmphillippo/multinma}
  \item Report bugs at \url{https://github.com/dmphillippo/multinma/issues}
}

}
\author{
\strong{Maintainer}: David M. Phillippo \email{david.phillippo@bristol.ac.uk} (\href{https://orcid.org/0000-0003-2672-7841}{ORCID})

Other contributors:
\itemize{
  \item Samuel J. Perren \email{samuel.perren@bristol.ac.uk} (\href{https://orcid.org/0009-0005-1921-6957}{ORCID}) [contributor]
  \item Niels Dunnewind (\href{https://orcid.org/0009-0009-8675-9109}{ORCID}) [contributor]
}

}
